declare const enum BitFlags {
    /**
     * this is a data */
    Data = 1,
    /**
     * this is a memo */
    Memo = 2,
    /**
     *
     */
    Dirty = 4,
    /**
     * current value is changed (so need to propagate)
     */
    Changed = 8,
    Evalutating = 16,
    /**
     * (computation) current value maybe dirty
     */
    MarkForCheck = 32,
    /**
     *
     */
    EffectWillCommit = 64,
    Effect = 256,
    /** only memo can be stale */
    Stale = 1024,
    ValueIsError = 2048,
    Managed = 8192,
    ManagedMemo = 8194,
    ValueWasError = 16384
}
interface InternalNode {
    flags: BitFlags;
}
interface Data<T = unknown> extends InternalNode {
    lo: ObserverLinked<Observer> | null;
    value: T | null;
    cp: (a: any, b: any) => boolean;
}
interface Observer extends InternalNode {
    dc: number;
    fs: SourceLinked<Data> | null;
    ls: SourceLinked<Data> | null;
}
interface Memo<T = unknown> extends Data<T>, Observer {
    c: Expression<T>;
    hs: T;
}
interface Reaction<T = unknown> extends Observer {
    e: () => T;
}
interface Expression<T> {
    (track: <D>(data: Data<D>) => D): T;
}
declare type SourceLinked<T> = {
    prev: SourceLinked<T> | null;
    next: SourceLinked<T> | null;
    source: T;
    observer_ref: ObserverLinked<any>;
};
declare type ObserverLinked<T> = {
    prev: ObserverLinked<T> | null;
    next: ObserverLinked<T> | null;
    observer: T;
};
declare function setData<T>(data: Data<T>, value: T): void;
declare function accessCurrent<T>(data: Data<T>, obs?: Memo<any> | null): T;
declare function cleanupObserver(memo: Observer): void;
declare function batch(fn: Function): any;
declare function createReaction(onCommit: () => void): Reaction;
declare function executeReaction<T>(reaction: Observer, fn: Expression<T>): T;
declare function createData<T>(value: T, comp?: (a: T, b: T) => boolean): Data<T>;
declare function createMemo<T>(fn: Expression<T>, comp?: (a: T, b: T) => boolean, initial?: T): Memo<T>;
export declare function takeControlOnCommit(controller: (continuation: () => void) => any): void;
export { BitFlags, Data, Memo, Reaction, batch, setData, accessCurrent as accessReferenceValue, createData, createMemo, createReaction, executeReaction, cleanupObserver, };
